function [structout, defaulted] = parsestruct(in, defaults)
% PARSESTRUCT Parse options in parameters string or structure, installing defaults
%   [STRUCTOUT, DEFAULTED] = PARSESTRUCT(IN, DEFAULTS)
% Inputs:
%   IN          - the input parameters, can be specified as structure or string,
%               see the below for the format of the string
%   DEFAULTS    - the default values for the parameters, as a structure
%               Every parameter should be on this structure, as it is used to infer the parameter
%               IDs. If a parameter has no default value, use an empty matrix
% Returns:
%   STRUCTOUT   - the parsed structure
%   DEFAULTED   - parameters that were not specified and thus took default values (cell array of
%               parameter IDs)
%
% If the configuration is given as a string, this string should contain assignments of the form
% name=value, separated by spaces. Only parameter names appearing in the defaults structure are
% considered. Any values that can be parsed by the function eval are supported, including at least
% numeric, string, (cell) array. Fields which are present on the string, but for which no value is
% assigned are treated as boolean "true" values. If arrays need to be entered, use commas and
% semicolons to separate the elements/rows, so that no spaces are needed.
%
% Example: the string 'alpha beta=2.5 gamma=value', given the field names 'alpha', 'beta', 'gamma',
% and 'delta', produces the structure:
%       alpha: 1        (boolean true)
%        beta: 2.500
%       gamma: 'value'
%       delta: 0       	(boolean false)


% Copyright:    2010 Delft Center for Systems & Control (TUDelft), and Lucian Busoniu
% Author:       Lucian Busoniu
% Version:      1.0
% 
% Please do not redistribute this code without permission from the authors. If you wish to
% redistribute, contact one of: Robert Babuska, Lucian Busoniu, Ivo Grondman, Gabriel Lopes

if isempty(defaults), error('Defaults are required to infer the field names'); end;

% if input is provided as string, parse it into a structure
if ischar(in),  
    structout = str2struct(in, fieldnames(defaults)); 
elseif isstruct(in),
    structout = in;
else
    error('IN should be string or structure');
end;

% install defaults for whatever hasn't been specified on the input
defaulted = {};
ids = fieldnames(defaults);
for i = 1:length(ids),
    if ~isfield(structout, ids{i})            % assign defaults
        structout.(ids{i}) = defaults.(ids{i});
        defaulted{end+1} = ids{i};
    end;
end;

end 

function structout = str2struct(strin, keywords)
% Obtain structure from easy-to-use string specification
%   STRUCTOUT = STR2STRUCT(STRIN, KEYWORDS)
% Parameters:
%   STRIN       - the configuration string
%   KEYWORDS    - keywords to search in the string, will become structure fields
%       Should be a cell array of strings, or a structure. In the second case the structure
%       field names will be used as keywords.
% Returns:
%   STRUCTOUT    - processed structure
%
% If STRIN is already a structure, the same structure is returned.

if isstruct(strin),           % assume already given as structure
    structout = strin;
    return;
end;
if isstruct(keywords),      % obtain field names
    keywords = fieldnames(keywords); 
end;

slen = length(strin);

structout = struct;
for i = 1:length(keywords),
    kw = keywords{i}; kwlen = length(kw);
    indices = strfind(strin, keywords{i});
    if isempty(indices), continue; end;      % no setting found
    accept = 0;
    for j = 1:length(indices),
        ix = indices(j); after = ix + kwlen; before = ix - 1;
        % accept only if entire keyword (i.e., preceded and followed by separator, 
        % or beginning / ending the string
        if ((after > slen) || (strin(after) == '=') || (strin(after) == ' ')) ...
                && ((before < 1) || (strin(before) == ' ')),
            accept = 1;
            break;
        end;
    end;
    if ~accept, continue; end;
    % if no value given, interpreted as boolean true
    if (after > slen) || (strin(after) == ' '),
        structout.(kw) = true;
        continue;
    end;
    % otherwise, parse after equal
    val = strtok(strin(after+1:end), ' ');
    % convert to various data types if possible
    % if it starts with a letter, it's a string
    if ~any(val(1) == ['a':'z' 'A':'Z']) && ~isempty(str2num(val)),      % try numeric
        val = str2num(val);
    elseif val(1) == '{',           % cell array
        try val = eval(val);
        catch   % no change, leave it as string
        end;
    end;
    structout.(kw) = val;
end;

end
